#include <avr/io.h>
#include <util/delay.h>



#define ende 1
#define weiter 0


#define MOUSE_PRODUCT_ID_REG					0x00
#define MOUSE_REVISION_ID_REG					0x01
#define MOUSE_MOTION_REG						0x02
#define MOUSE_DELTA_X_REG						0x03
#define MOUSE_DELTA_Y_REG						0x04
#define MOUSE_SQUAL_REG							0x05
#define MOUSE_PIXEL_SUM_REG						0x06
#define MOUSE_MAXIMUM_PIXEL_REG 				0x07
#define MOUSE_CONFIGURATION_BITS_REG			0x0a
#define MOUSE_EXTENDED_CONFIG_REG				0x0b
#define MOUSE_DATA_OUT_LOWER_REG				0x0c
#define MOUSE_DATA_OUT_UPPER_REG				0x0d
#define MOUSE_SHUTTER_LOWER_REG					0x0e
#define MOUSE_SHUTTER_UPPER_REG					0x0f
#define MOUSE_FRAME_PERIOD_LOWER_REG			0x10
#define MOUSE_FRAME_PERIOD_UPPER_REG			0x11
#define MOUSE_MOTION_CLEAR_REG					0x12
#define MOUSE_FRAME_CAPTURE_REG					0x13
#define MOUSE_SROM_ENABLE_REG					0x14
#define MOUSE_FRAME_PERIOD_MAX_BOUND_LOWER_REG	0x19
#define MOUSE_FRAME_PERIOD_MAX_BOUND_UPPER_REG	0x1a
#define MOUSE_FRAME_PERIOD_MIN_BOUND_LOWER_REG	0x1b
#define MOUSE_FRAME_PERIOD_MIN_BOUND_UPPER_REG	0x1c
#define MOUSE_SHUTTER_MAX_BOUND_LOWER_REG		0x1d
#define MOUSE_SHUTTER_MAX_BIUND_UPPER_REG		0x1e
#define MOUSE_SROM_ID_REG						0x1f
#define MOUSE_OBSERVATION_REG					0x3d
#define MOUSE_INVERSE_PRODUCT_ID				0x3f
#define MOUSE_PIXEL_BURST_REG					0x40
#define MOUSE_MOTION_BURST_REG					0x50
#define MOUSE_SROM_LOAD_REG						0x60



void mouse_write(uint8_t adr, uint8_t data)
{
	SPI_transmit(adr|=0x80, weiter);					// MSB muss 1 sein
	SPI_transmit(data, ende);							// Funktion sendet nur Adresse und 1 Byte, danach wird Uebertragung geschlossen
	_delay_us(100);
}

uint8_t mouse_read(uint8_t adr)
{
	SPI_transmit(adr,weiter);							// Funktion sendet nur Adresse und empfaengt 1 Byte, danach wird Uebertragung geschlossen
	// _delay_us(100);
	return SPI_receive(ende);
}

void motion_read(void)
{
	uint8_t Feld[6];
	
	SPI_transmit(MOUSE_MOTION_BURST_REG,weiter);				// Das Lesen dieses Register startet die Uebertragung
	
	_delay_us(75);										// Mindest-Wartezeit: 75s
	
	for (uint8_t i=0;i<6;i++)							// 7 Register werden ausgelesen
	{
		Feld[i]=SPI_receive(weiter);
	}
	Feld[6]=SPI_receive(ende);
	_delay_us(5);										// mind. 4s, damit der Burst-Modus verlassen werden kann
	
	for (uint8_t i=0;i<7;i++)
	{
		USART_Transmit(Feld[i]);
	}
}

uint16_t frame_period(void)								// Gibt Frame-Capture-Zeit zurueck
{
	SPI_transmit(MOUSE_FRAME_PERIOD_UPPER_REG,weiter);
	uint8_t upper=SPI_receive(ende);					// zwei Register sind erforderlich
	SPI_transmit(MOUSE_FRAME_PERIOD_LOWER_REG,weiter);
	uint8_t lower=SPI_receive(ende);
	uint16_t gesamt=(upper<<8)|(lower<<8);
	return(1/gesamt);
}

void mouse_image_prepare(void)
{
	SPI_transmit(MOUSE_FRAME_CAPTURE_REG|0x80,weiter);		// Frame Daten werden gespeichert
	SPI_transmit(0xFF,ende);
	_delay_us(10+3*frame_period());						// tCapture (s. Datenblatt)
}

void mouse_image_read(void)
{
	SPI_transmit(MOUSE_PIXEL_BURST_REG,weiter);				// Auslesen der Werte beginnt mit dem Lesen das PIXEL_BURST_REGISTERS
	_delay_us(55);										// Mindest-Wartezeit: 50s
	uint8_t* pBuffer=malloc(900);						// Ein Frame=900 Byte; Sensor sendet aber 1 2/3 Frame - der Rest wird nicht beachtet
	unsigned char data;
	
	for(uint16_t i=0;i<900;i++)
	{
		pBuffer[i]=SPI_receive(weiter);
	}
	for(uint16_t i=0;i<634;i++)
	{
		data=SPI_receive(weiter);
	}
	data=SPI_receive(ende);
	_delay_us(10);
	for(uint16_t i=0;i<900;i++)
	{
		USART_Transmit(pBuffer[i]);
	}
	free(pBuffer);
}
